#===============================================================================
# Copyright Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#===============================================================================

# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

# -- Project information -----------------------------------------------------

project = 'oneDPL Documentation'
copyright = 'Intel Corporation'
author = 'Intel'

# The full version, including alpha/beta/rc tags
release = '2022.9.0'

rst_epilog = """
.. include:: /variables.txt
.. include:: /common_cross_document_links.txt
"""

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx-prompt',
    'sphinx_substitution_extensions'
	]

docbundle_settings = {
    'csv_dir':'csv_dir'
}

# The master toctree document.
master_doc = 'index'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.

# html_theme = 'alabaster'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']
html_css_files = [
    'custom.css',
]
html_logo = '_static/oneAPI-rgb-rev-100.png'
html_favicon = '_static/favicons.png'
#latex_logo = '_static/oneAPI-rgb-3000.png'
html_show_sourcelink = False
#html_js_files = ['custom.js']

# html_context = {
#     'css_files': [
#         '_static/style.css',  # override wide tables in RTD theme
#         ],
#     }

# html_theme = 'otc_tcs_sphinx_theme'
# html_theme_path = ['_themes']

# import sphinx_rtd_theme
html_theme = 'sphinx_book_theme'
#html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
#if on_rtd:
#    using_rtd_theme = True

# Theme options
html_theme_options = {
    # 'typekit_id': 'hiw1hhg',
    # 'analytics_id': '',
    # 'sticky_navigation': True,  # Set to False to disable the sticky nav while scrolling.
    #'logo_only': False,  # if we have a html_logo below, this shows /only/ the logo with no title text
    #'collapse_navigation': False,  # Collapse navigation (False makes it tree-like)
    #'navigation_depth': 4  # Depth of the headers shown in the navigation bar
    #'display_version': True,  # Display the docs version
    'repository_url': 'https://github.com/uxlfoundation/oneDPL',
    'path_to_docs': 'documentation/library_guide',
    'use_issues_button': True,
    'use_edit_page_button': True,
    'repository_branch': 'main',
    'extra_footer': '<p align="right"><a data-cookie-notice="true" href="https://www.intel.com/content/www/us/en/privacy/intel-cookie-notice.html">Cookies</a></p>'
}



import argparse
import re

from collections import Counter


def parse_arguments():
    na_value = "N/A"
    parser = argparse.ArgumentParser(description="Analyze build logs and ctest output, and generate a summary.")
    parser.add_argument("--build-log", required=True, help="Path to the build log file.")
    parser.add_argument("--ctest-log", required=True, help="Path to the ctest log file.")
    parser.add_argument("--output-file", required=True, help="Path to the output file.")
    parser.add_argument("--os", default=na_value, help="Operating System.")
    parser.add_argument("--compiler-version", default=na_value, help="Version of the compiler.")
    parser.add_argument("--cmake-version", default=na_value, help="Version of CMake.")
    parser.add_argument("--cpu-model", default=na_value, help="CPU model.")
    return parser.parse_args()


def read_file(file_path):
    with open(file_path, 'r', encoding="utf8") as f:
        return f.read()


def generate_environment_table(os_info, compiler_version, cmake_version, cpu_model):
    table = []
    table.append( "| Environment Parameter | Value               |")
    table.append( "|-----------------------|---------------------|")
    table.append(f"| OS                    | {os_info}           |")
    table.append(f"| Compiler Ver.         | {compiler_version}  |")
    table.append(f"| CMake Ver.            | {cmake_version}     |")
    table.append(f"| CPU Model             | {cpu_model}         |")
    return "\n".join(table)


def generate_warning_table(build_log_content):
    warning_regex = re.compile(
        r"""
            \[(\-W[a-zA-Z0-9\-=]+)\] |  # GCC/Clang warnings: "[-W<some-flag>]" or "[-W<some-flag>=<value>]"
            ((?:STL|C|D|LNK)\d{4})     # MSVC warnings: "<STL|C|D|LNK>xxxx"
        """,
        re.VERBOSE
    )
    warnings = tuple(
        match.group(1) or match.group(2)
        for match in warning_regex.finditer(build_log_content)
    )
    warning_histogram = Counter(warnings)

    warning_examples = {}
    for w in warning_histogram:
        matches = tuple(line for line in build_log_content.splitlines() if w in line)
        # Prioritize warnigs from the core library ("include" is expected in the message as a part of the path)
        lib_match = next((m for m in matches if "include" in m), None)
        first_match = matches[0]
        warning_examples[w] = lib_match or first_match

    table = []
    table.append("| Warning Type   | Count | Message example |")
    table.append("|----------------|-------|-----------------|")
    for warning, count in warning_histogram.items():
        example = warning_examples[warning]
        table.append(f"| {warning} | {count} | {example} |")
    return "\n".join(table)


def generate_ctest_table(ctest_log_content):
    # No need to parse the data into Markdown table: it is already in a readable pseudo-table format
    result_lines = re.findall(r".*Test\s*#.*sec.*", ctest_log_content)
    code_block = ["```"] + result_lines + ["```"]
    return "\n".join(code_block)


def extract_ctest_summary(ctest_log_content):
    match = re.search(r".*tests passed.*tests failed.*", ctest_log_content)
    if match is None:
        return ""
    else:
        return match.group(0)


def combine_tables(environment_table, warning_table, ctest_table, ctest_summary):
    # Make the CTest summary collapsible since it can be long
    title = f"<summary><b>CTest: {ctest_summary} (expand for details)</b></summary>"
    collapsible_ctest_table = f"<details>\n{title}\n\n{ctest_table}\n\n</details>"
    # Additional empty line to separate the tables
    return "\n\n".join([environment_table, warning_table, collapsible_ctest_table])


if __name__ == "__main__":
    args = parse_arguments()
    build_log_content = read_file(args.build_log)
    ctest_log_content = read_file(args.ctest_log)

    environment_table = generate_environment_table(args.os, args.compiler_version, args.cmake_version, args.cpu_model)
    warning_table = generate_warning_table(build_log_content)
    ctest_table = generate_ctest_table(ctest_log_content)
    ctest_summary = extract_ctest_summary(ctest_log_content)
    summary = combine_tables(environment_table, warning_table, ctest_table, ctest_summary)

    with open(args.output_file, 'w', encoding="utf-8") as f:
        f.write(summary)
